@@ -178,7 +178,6 @@ def wheel_files_pyside_essentials() -> list[ModuleData]:
         module_QtSvg(),
         module_QtSvgWidgets(),
         module_QtUiTools(),
-        module_QtExampleIcons(),
         # Only for plugins
         module_QtWayland(),
         # there are no bindings for these modules, but their binaries are
@@ -319,6 +318,10 @@ def module_QtCore() -> ModuleData:
     data.qtlib.append("libicuuc*")
     data.qtlib.append("libicuio*")
 
+    # add the include folders for libpyside binaries
+    # this is useful for downstream cmake projects like QtBridges
+    data.include.append("pyside6/*.h")
+
     return data
 
 
@@ -327,10 +330,12 @@ def module_QtGui() -> ModuleData:
     _typesystems = [
         "gui_common.xml",
         "typesystem_gui_common.xml",
+        "typesystem_gui_nativeinterface.xml",
         "typesystem_gui_mac.xml",
         "typesystem_gui_win.xml",
         "typesystem_gui_x11.xml",
-        "typesystem_gui_rhi.xml"
+        "typesystem_gui_rhi.xml",
+        "typesystem_gui_wayland.xml"
     ]
 
     _metatypes = [
@@ -732,6 +737,8 @@ def module_Qt3DInput() -> ModuleData:
 
 def module_Qt3DLogic() -> ModuleData:
     data = ModuleData("3DLogic", qml=["Qt3D/Logic"])
+    json_data = get_module_json_data("3DLogic")
+    data.plugins = get_module_plugins(json_data)
 
     return data
 
@@ -766,10 +773,11 @@ def module_QtQuick3D() -> ModuleData:
         "libQt63DQuick",
         "libQt63DQuickAnimation",
         "libQt63DQuickExtras",
-        "libQt63DQuickExtras",
+        "libQt63DQuickLogic",
         "libQt63DQuickInput",
         "libQt63DQuickRender",
         "libQt63DQuickScene2D",
+        "libQt63DQuickScene3D",
         "libQt6Quick3DXr",
     ]
 
@@ -1073,11 +1081,6 @@ def module_QtAsyncio() -> ModuleData:
     return data
 
 
-def module_QtExampleIcons() -> ModuleData:
-    data = ModuleData("ExampleIcons")
-    return data
-
-
 def module_QtWebView() -> ModuleData:
     data = ModuleData("WebView")
     json_data = get_module_json_data("WebView")
