@@ -2,12 +2,13 @@
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "class_property.h"
-#include "pysidestaticstrings.h"
 #include "feature_select.h"
 
+#include <basewrapper.h>
 #include <pep384ext.h>
-#include <shiboken.h>
 #include <sbkstaticstrings.h>
+#include <sbktypefactory.h>
+#include <signature.h>
 
 extern "C" {
 
@@ -79,7 +80,7 @@ PyTypeObject *PyClassPropertyType_TypeF()
 // We call `__init__` while pretending to be a PyProperty_Type instance.
 static int PyClassProperty_tp_init(PyObject *self, PyObject *args, PyObject *kwargs)
 {
-    auto hold = Py_TYPE(self);
+    auto *hold = Py_TYPE(self);
     self->ob_type = &PyProperty_Type;
     auto ret = PepExt_Type_GetInitSlot(&PyProperty_Type)(self, args, kwargs);
     self->ob_type = hold;
@@ -126,7 +127,7 @@ static int SbkObjectType_meta_setattro(PyObject *obj, PyObject *name, PyObject *
 {
     // Use `_PepType_Lookup()` instead of `PyObject_GetAttr()` in order to get the raw
     // descriptor (`property`) instead of calling `tp_descr_get` (`property.__get__()`).
-    auto type = reinterpret_cast<PyTypeObject *>(obj);
+    auto *type = reinterpret_cast<PyTypeObject *>(obj);
     PySide::Feature::Select(type);
     PyObject *descr = _PepType_Lookup(type, name);
 
@@ -134,7 +135,7 @@ static int SbkObjectType_meta_setattro(PyObject *obj, PyObject *name, PyObject *
     //   1. `Type.class_prop = value`              --> descr_set: `Type.class_prop.__set__(value)`
     //   2. `Type.class_prop = other_class_prop`   --> setattro:  replace existing `class_prop`
     //   3. `Type.regular_attribute = value`       --> setattro:  regular attribute assignment
-    const auto class_prop = reinterpret_cast<PyObject *>(PyClassProperty_TypeF());
+    auto *class_prop = reinterpret_cast<PyObject *>(PyClassProperty_TypeF());
     const auto call_descr_set = descr && PyObject_IsInstance(descr, class_prop)
                                 && !PyObject_IsInstance(value, class_prop);
     if (call_descr_set) {
@@ -153,13 +154,13 @@ namespace PySide::ClassProperty {
 
 static const char *PyClassProperty_SignatureStrings[] = {
     "PySide6.QtCore.PyClassProperty(cls,"
-        "fget:typing.Optional[typing.Callable[[typing.Any],typing.Any]]=None,"
-        "fset:typing.Optional[typing.Callable[[typing.Any,typing.Any],None]]=None,"
-        "fdel:typing.Optional[typing.Callable[[typing.Any],None]]=None,"
+        "fget:typing.Optional[collections.abc.Callable[[typing.Any],typing.Any]]=None,"
+        "fset:typing.Optional[collections.abc.Callable[[typing.Any,typing.Any],None]]=None,"
+        "fdel:typing.Optional[collections.abc.Callable[[typing.Any],None]]=None,"
         "doc:typing.Optional[str]=None)",
-    "PySide6.QtCore.PyClassProperty.getter(cls,fget:typing.Callable[[typing.Any],typing.Any])->PySide6.QtCore.PyClassProperty",
-    "PySide6.QtCore.PyClassProperty.setter(cls,fset:typing.Callable[[typing.Any,typing.Any],None])->PySide6.QtCore.PyClassProperty",
-    "PySide6.QtCore.PyClassProperty.deleter(cls,fdel:typing.Callable[[typing.Any],None])->PySide6.QtCore.PyClassProperty",
+    "PySide6.QtCore.PyClassProperty.getter(cls,fget:collections.abc.Callable[[typing.Any],typing.Any])->PySide6.QtCore.PyClassProperty",
+    "PySide6.QtCore.PyClassProperty.setter(cls,fset:collections.abc.Callable[[typing.Any,typing.Any],None])->PySide6.QtCore.PyClassProperty",
+    "PySide6.QtCore.PyClassProperty.deleter(cls,fdel:collections.abc.Callable[[typing.Any],None])->PySide6.QtCore.PyClassProperty",
     nullptr}; // Sentinel
 
 void init(PyObject *module)
@@ -167,12 +168,13 @@ void init(PyObject *module)
     PyTypeObject *type = SbkObjectType_TypeF();
     type->tp_setattro = SbkObjectType_meta_setattro;
 
-    if (InitSignatureStrings(PyClassProperty_TypeF(), PyClassProperty_SignatureStrings) < 0)
+    auto *classPropertyType = PyClassProperty_TypeF();
+    if (InitSignatureStrings(classPropertyType, PyClassProperty_SignatureStrings) < 0)
         return;
 
-    Py_INCREF(PyClassProperty_TypeF());
-    auto classproptype = reinterpret_cast<PyObject *>(PyClassProperty_TypeF());
-    PyModule_AddObject(module, "PyClassProperty", classproptype);
+    auto *obClassPropertyType = reinterpret_cast<PyObject *>(classPropertyType);
+    Py_INCREF(obClassPropertyType);
+    PepModule_AddType(module, classPropertyType);
 }
 
 } // namespace PySide::ClassProperty
