@@ -15,6 +15,7 @@
 #include <bindingmanager.h>
 #include <gilstate.h>
 #include <sbkconverter.h>
+#include <sbkpep.h>
 #include <sbkstring.h>
 #include <sbkstaticstrings.h>
 #include <sbkerrors.h>
@@ -24,9 +25,12 @@
 #include <QtCore/qcoreevent.h>
 #include <QtCore/qdebug.h>
 #include <QtCore/qhash.h>
+#include <QtCore/qmetatype.h>
 #include <QtCore/qscopedpointer.h>
 
+#include <climits>
 #include <memory>
+#include <utility>
 
 using namespace Qt::StringLiterals;
 
@@ -36,6 +40,8 @@ using namespace Qt::StringLiterals;
 
 static PyObject *metaObjectAttr = nullptr;
 
+static int pyObjectWrapperMetaTypeId = QMetaType::UnknownType;
+
 static void destroyMetaObject(PyObject *obj)
 {
     void *ptr = PyCapsule_GetPointer(obj, nullptr);
@@ -125,6 +131,17 @@ PyObjectWrapper::PyObjectWrapper(const PyObjectWrapper &other)
     Py_XINCREF(m_me);
 }
 
+PyObjectWrapper::PyObjectWrapper(PyObjectWrapper &&other) noexcept
+ : m_me{std::exchange(other.m_me, nullptr)}
+{
+}
+
+PyObjectWrapper &PyObjectWrapper::operator=(PyObjectWrapper &&other) noexcept
+{
+    m_me = std::exchange(other.m_me, nullptr);
+    return *this;
+}
+
 PyObjectWrapper::~PyObjectWrapper()
 {
     // Check that Python is still initialized as sometimes this is called by a static destructor
@@ -156,6 +173,10 @@ PyObjectWrapper::operator PyObject *() const
     return m_me;
 }
 
+int PyObjectWrapper::metaTypeId()
+{
+    return pyObjectWrapperMetaTypeId;
+}
 
 int PyObjectWrapper::toInt() const
 {
@@ -221,7 +242,29 @@ QDataStream &operator>>(QDataStream &in, PyObjectWrapper &myObj)
     return in;
 }
 
-};
+PYSIDE_API QDebug operator<<(QDebug debug, const PyObjectWrapper &myObj)
+{
+    QDebugStateSaver saver(debug);
+    debug.noquote();
+    debug.nospace();
+    // Do not repeat the type name as it is typically called from the QVariant debug
+    // operator, which outputs the type.
+    debug << '<';
+    if (PyObject *ob = myObj) {
+        const auto refs = Py_REFCNT(ob);
+        debug << PepType_GetFullyQualifiedNameStr(Py_TYPE(ob)) << " at " << ob;
+        if (refs == UINT_MAX) // _Py_IMMORTAL_REFCNT
+            debug << ", immortal";
+        else
+            debug << ", refs=" << refs;
+    } else {
+        debug << '0';
+    }
+    debug << '>';
+    return debug;
+}
+
+} // namespace PySide
 
 using namespace PySide;
 
@@ -238,19 +281,11 @@ struct SignalManagerPrivate
 SignalManager::QmlMetaCallErrorHandler
     SignalManagerPrivate::m_qmlMetaCallErrorHandler = nullptr;
 
-static void PyObject_PythonToCpp_PyObject_PTR(PyObject *pyIn, void *cppOut)
+static PyObject *CopyCppToPythonPyObject(const void *cppIn)
 {
-    *reinterpret_cast<PyObject **>(cppOut) = pyIn;
-}
-static PythonToCppFunc is_PyObject_PythonToCpp_PyObject_PTR_Convertible(PyObject * /* pyIn */)
-{
-    return PyObject_PythonToCpp_PyObject_PTR;
-}
-static PyObject *PyObject_PTR_CppToPython_PyObject(const void *cppIn)
-{
-    auto *pyOut = reinterpret_cast<PyObject *>(const_cast<void *>(cppIn));
-    if (pyOut)
-        Py_INCREF(pyOut);
+    const auto *wrapper = reinterpret_cast<const PyObjectWrapper *>(cppIn);
+    PyObject *pyOut = *wrapper;
+    Py_XINCREF(pyOut);
     return pyOut;
 }
 
@@ -260,13 +295,16 @@ void SignalManager::init()
     using namespace Shiboken;
 
     // Register PyObject type to use in queued signal and slot connections
-    qRegisterMetaType<PyObjectWrapper>("PyObject");
+    pyObjectWrapperMetaTypeId = qRegisterMetaType<PyObjectWrapper>("PyObject");
     // Register QVariant(enum) conversion to QVariant(int)
     QMetaType::registerConverter<PyObjectWrapper, int>(&PyObjectWrapper::toInt);
 
-    SbkConverter *converter = Shiboken::Conversions::createConverter(&PyBaseObject_Type, nullptr);
-    Shiboken::Conversions::setCppPointerToPythonFunction(converter, PyObject_PTR_CppToPython_PyObject);
-    Shiboken::Conversions::setPythonToCppPointerFunctions(converter, PyObject_PythonToCpp_PyObject_PTR, is_PyObject_PythonToCpp_PyObject_PTR_Convertible);
+    // Register a shiboken converter for PyObjectWrapper->Python (value conversion).
+    // Python->PyObjectWrapper is not registered since the converters do not work for
+    // non-SbkObject types (falling back to plain pointer pass through).
+    // This conversion needs to be done manually via QVariant.
+    SbkConverter *converter = Shiboken::Conversions::createConverter(&PyBaseObject_Type,
+                                                                     CopyCppToPythonPyObject);
     Shiboken::Conversions::registerConverterName(converter, "PyObject");
     Shiboken::Conversions::registerConverterName(converter, "object");
     Shiboken::Conversions::registerConverterName(converter, "PyObjectWrapper");
@@ -315,6 +353,18 @@ void SignalManager::handleMetaCallError()
     Py_SetRecursionLimit(reclimit);
 }
 
+const char *metaObjectCallName(QMetaObject::Call call)
+{
+    static const char *names[] = {
+        "InvokeMetaMethod", "ReadProperty", "WriteProperty", "ResetProperty",
+        "CreateInstance", "IndexOfMethod", "RegisterPropertyMetaType",
+        "RegisterMethodArgumentMetaType", "BindableProperty", "CustomCall",
+        "ConstructInPlace"};
+    constexpr size_t count = sizeof(names)/sizeof(names[0]);
+    static_assert(QMetaObject::ConstructInPlace == count - 1);
+    return call >= 0 && call < count ? names[call] : "<unknown>";
+}
+
 // Handler for QMetaObject::ReadProperty/WriteProperty/ResetProperty:
 int SignalManagerPrivate::qtPropertyMetacall(QObject *object,
                                              QMetaObject::Call call,
@@ -347,25 +397,20 @@ int SignalManagerPrivate::qtPropertyMetacall(QObject *object,
     if (PyErr_Occurred()) {
         // PYSIDE-2160: An unknown type was reported. Indicated by StopIteration.
         if (PyErr_ExceptionMatches(PyExc_StopIteration)) {
-            PyObject *excType{};
-            PyObject *excValue{};
-            PyObject *excTraceback{};
-            PyErr_Fetch(&excType, &excValue, &excTraceback);
+            Shiboken::Errors::Stash errorStash;
             bool ign = call == QMetaObject::WriteProperty;
             PyErr_WarnFormat(PyExc_RuntimeWarning, 0,
                 ign ? "Unknown property type '%s' of QObject '%s' used in fset"
                     : "Unknown property type '%s' of QObject '%s' used in fget with %R",
-                pp->d->typeName.constData(), metaObject->className(), excValue);
+                pp->d->typeName.constData(), metaObject->className(), errorStash.getException());
             if (PyErr_Occurred())
                 Shiboken::Errors::storeErrorOrPrint();
-            Py_DECREF(excType);
-            Py_DECREF(excValue);
-            Py_XDECREF(excTraceback);
+            errorStash.release();
             return result;
         }
 
         qWarning().noquote().nospace()
-            << "An error occurred executing the property metacall " << call
+            << "An error occurred executing the property metacall " << metaObjectCallName(call)
             << " on property \"" << mp.name() << "\" of " << object;
         handleMetaCallError(object, &result);
     }
@@ -705,6 +750,12 @@ int SignalManager::registerMetaMethodGetIndexBA(QObject* source, const QByteArra
 
 const QMetaObject *SignalManager::retrieveMetaObject(PyObject *self)
 {
+#ifdef Py_GIL_DISABLED
+    // PYSIDE-2221: When working with disable-gil, it seems to be necessary
+    //              to hold the GIL. Maybe that is harmless here (check later).
+    // Thanks to Sam Gross who fixed most errors by pointing this out.
+    Shiboken::GilState gil;
+#endif
     // PYSIDE-803: Avoid the GIL in SignalManager::retrieveMetaObject
     // This function had the GIL. We do not use the GIL unless we have to.
     // metaBuilderFromDict accesses a Python dict, but in that context there
