@@ -37,12 +37,19 @@ LIBSHIBOKEN_API void resolveLazyClasses(PyObject *module);
 LIBSHIBOKEN_API PyObject *import(const char *moduleName);
 
 /**
- *  Creates a new Python module named \p moduleName using the information passed in \p moduleData.
- *  In fact, \p moduleData expects a "PyMethodDef *" object, but that's for Python 2. A "void*"
- *  was preferred to make this work with future Python 3 support.
+ *  Creates a new Python module named \p moduleName using the information passed in \p moduleData
+ *  and calls exec() on it.
  *  \returns a newly created module.
  */
-LIBSHIBOKEN_API PyObject *create(const char *moduleName, void *moduleData);
+[[deprecated]] LIBSHIBOKEN_API PyObject *create(const char *moduleName, PyModuleDef *moduleData);
+
+/// Creates a new Python module named \p moduleName using the information passed in \p moduleData.
+/// exec() is not called (Support for Nuitka).
+/// \returns a newly created module.
+LIBSHIBOKEN_API PyObject *createOnly(const char *moduleName, PyModuleDef *moduleData);
+
+/// Executes a module (multi-phase initialization helper)
+LIBSHIBOKEN_API void exec(PyObject *module);
 
 using TypeCreationFunction = PyTypeObject *(*)(PyObject *module);
 
@@ -84,6 +91,11 @@ LIBSHIBOKEN_API void registerTypeConverters(PyObject *module, SbkConverter **con
  */
 LIBSHIBOKEN_API SbkConverter **getTypeConverters(PyObject *module);
 
+/**
+ *  Replace the dictionary of a module. This allows to use `__missing__`.
+ */
+LIBSHIBOKEN_API bool replaceModuleDict(PyObject *module, PyObject *modClass, PyObject *dict);
+
 } // namespace Shiboken::Module
 
 #endif // SBK_MODULE_H
